/*  audio.c - audio IO functions
 *  Copyright (C) 1999  Charles <int@linuxcore.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * $Id: audio.c,v 1.3 1999/07/27 17:02:29 int Exp $
 */

/*#include <config.h>
*/
#include <esd.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/poll.h>

#include "audio.h"

#define DEF_BUF_SIZE 1024


static char *host = NULL;           // esound host
static int sock = -1;               // esound socket
static char *audiobuf[2];           // audio data buffer
static int buflen = 0;              // number of bytes currently in buffer
static int bufsize = DEF_BUF_SIZE;  // size of the audio data buffer
static int status = 0;


static int audio_init_esd(int size, int rate)
{
	int bufsize = size;
	esd_format_t format = 0;
	
	format = ESD_BITS8 | ESD_MONO | ESD_STREAM | ESD_PLAY;
	printf("ESD: connect(%s)\n", host);
	// ESD seems to always try to connect to a local unix socket
	// first, even if the host is remote.
	sock = esd_monitor_stream(format, rate, host, "vsa_applet");
	if(sock < 0)
		return -1;

	audiobuf[0] = (char *)malloc(bufsize);
	audiobuf[1] = (char *)malloc(bufsize);
	audio_reset();

	return sock;
}


int audio_start(char *hname, int port, int s, int r)
{
	if(hname)
	{
		if(port > 0)
		{
			host = (char *)malloc(strlen(hname) + 10);
			sprintf(host, "%s:%d", hname, port);
		}	
		else
			host = strdup(hname);
	}

	return audio_init_esd(s, r);
}

void audio_stop()
{
	if(sock != -1)
		close(sock);
	sock = -1;
	free(host);
	host = NULL;
	if(audiobuf[0] != NULL)
		free(audiobuf[0]);
	if(audiobuf[1] != NULL)
		free(audiobuf[1]);
	audiobuf[0] = audiobuf[1] = NULL;
	audio_reset();
}

// put ESD in standby mode
int audio_suspend()
{
	int fd;

	fd = esd_open_sound(host);
	if(fd < 0)
		return 0;
	esd_standby(fd);
	esd_close(fd);
	return 1;
}


static void start_esd()
{
	if(!fork())
	{
		setsid();
		execlp("esd", NULL);
		printf("failed to exec esd\n");
		_exit(1);
	}
}

// resume ESD
int audio_resume()
{
	static int retry = 0;
	int fd;

	fd = esd_open_sound(host);
	if(fd < 0)
	{
		if(retry)
		{
			retry = 0;
			return 0;
		}
		//FIXME
		if(host || strcmp(host, "localhost"))
		{
			// Don't try start esd if we're connecting
			// to a remote host
			return 0;
		}
		start_esd();
		retry = 1;
		return audio_resume();
	}
	esd_resume(fd);
	esd_close(fd);
	return 1;
}


AUDIO_ST audio_status()
{
	int fd;
	AUDIO_ST st;

	fd = esd_open_sound(host);
	if(fd < 0)
		return AUDIO_ST_ERROR;

	switch(esd_get_standby_mode(fd))
	{
	case ESM_ON_STANDBY:
		st = AUDIO_ST_STANDBY;
		break;
	case ESM_ON_AUTOSTANDBY:
	case ESM_RUNNING:
		st = AUDIO_ST_OK;
		break;
	default:
		st = AUDIO_ST_ERROR;
		break;
	}
	return st;
}

int audio_ok()
{
	return status < 0 ? 0 : 1;
}

void audio_reset()
{
	buflen = 0;
	status = 0;
}


static int do_read(char *buf)
{
	struct pollfd pfd;
	int ret;

	pfd.fd = sock;
	pfd.events = POLLIN;
	if(poll((struct pollfd *)&pfd, 1, 10) <= 0)
		return -1;

        ret = read(sock, (char *)&buf[buflen], bufsize - buflen);
        if(ret <= 0)
	{
		status = -2;
                return status;
	}

	buflen += ret;

	if(buflen < bufsize)
		return -3;
	buflen = 0;

	return bufsize;
}


char *audio_read_buffer()
{
	static int which = 0;
	char *buf;
	int ret;
	
	buf = audiobuf[which];
	ret = do_read(buf);

	if(ret == 0)
		which = !which;
	return ret < 0 ? NULL : buf;
}
