/*  fft.c - FFT functions
 *  Copyright (C) 1999  Charles <int@linuxcore.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * $Id: fft.c,v 1.5 1999/07/25 09:17:27 int Exp $
 */

/*#include <config.h>
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include <rfftw.h>

static fftw_real *in, *out;
static rfftw_plan plan = 0;

static int fftsize = 0;
static int rate = 0;


void fft_start(int size, int r)
{
	fftsize = size;
	rate    = r;

	in  = (fftw_real *)malloc(sizeof(fftw_real) * size);
        out = (fftw_real *)malloc(sizeof(fftw_real) * size);

        plan = rfftw_create_plan(size, FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE);
}


void fft_stop()
{
	if(plan != 0)
	{
		free(in);
		free(out);
		rfftw_destroy_plan(plan);
		plan = 0;
	}
}


static double *fft_process(unsigned char *buf)
{
        int i;

	for(i = 0; i < fftsize; i++) 
		/* in[i] = ((double)buf[i]-128.0) * 
			(0.54 - 0.46 * cos(2.0 * M_PI * (double)i/(fftsize-1)));
		*/ in[i] = ((double)buf[i]-128.0) / 1.0;

	rfftw_one(plan, in, out);

	return (double *)out;
}


double *fft_get_ps(unsigned char *buf)
{
	int i;
	double *data;

	data = fft_process(buf);

        data[0] *= data[0];  // dc
	data[fftsize / 2] *= data[fftsize / 2]; 

	for(i = 1; i < (fftsize + 1) / 2; i++)
		data[i] = fabs(sqrt(data[i] * data[i] +
				    data[fftsize-i] * data[fftsize-i]));

	return (double *)data;
}
