#include <gtk/gtk.h>
#include "audio.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

static GtkWidget *progress[8];
static GtkObject *sens_adj[8];
static GtkObject *light_adj;

static int timeout;
static int idle_id = -1;
static int next_buf = -1;
static unsigned char *audio_buf;

static int dammit = -1;

double *fft_get_ps( unsigned char *buf );
void fft_start(int, int);
void fft_stop();

gint delete_event( GtkWidget *widget, GdkEvent *event, gpointer data ) {
	return FALSE;
}

gint destroy( GtkWidget *widget, GdkEvent *event, gpointer data ) {
	gtk_main_quit();
	return 0;
}

static gint update_audio( gpointer data ) {
	static unsigned char *prev = NULL;
	static unsigned int last_block = 0;
	unsigned char *buf;

	buf = audio_read_buffer();
	if( buf ) {
		audio_buf = prev;
		prev = buf;
		next_buf = 0;
		last_block = time( NULL );
	}
	else if( next_buf ) {
		audio_buf = NULL;
	}
	if( !audio_ok() ) {
		printf( "Audio error!\n" );
	}
	return 1;

}

gint do_analyze( gpointer data ) {
	double *fftout = NULL;
	unsigned int n_lights = GTK_ADJUSTMENT(light_adj)->value;
	unsigned char mask = 0;

	if( audio_buf ) {
		fftout = fft_get_ps( audio_buf );
	}

 	if( fftout ) {
		int i;
		for( i = 0; i < n_lights; i++ ) {
			double sens;
			double val = fftout[ (1024/(n_lights + 1)) * (i+1) ] ;
			val = val / 100.0;
			val = val > 1.0 ? 0: val;
			val = val < 0.0 ? 0: val;
			gtk_progress_bar_update( GTK_PROGRESS_BAR( progress[i] ), val );
			sens = (100.0-GTK_ADJUSTMENT(sens_adj[i])->value)/100.0;
			if( val > sens ) {
				mask |= 1 << i;
			} 
		}
		write( dammit, &mask, 1 );
	}

 	return 1;	
}

int main( int argc, char *argv[] ) {
	GtkWidget *window;
	GtkWidget *sensitivity[8];
	GtkWidget *pack1;
	GtkWidget *pack2;
	GtkWidget *light;
	
	int i;

	gtk_init( &argc, &argv );

	window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	pack1 = gtk_hbox_new(FALSE, 2);

	light_adj = gtk_adjustment_new( 4, 1, 8, 1, 1, 0 );
	light = gtk_hscale_new( GTK_ADJUSTMENT(light_adj) );
	gtk_scale_set_digits( GTK_SCALE( light ), 0 );
	pack2 = gtk_vbox_new( FALSE, 2);

	gtk_box_pack_start( GTK_BOX( pack2 ), light, FALSE, FALSE, FALSE );
	gtk_widget_show( light );

	for( i = 0; i < 8; i++ ) {
		sens_adj[i] = gtk_adjustment_new( 90, 0, 100, 1, 1, 1 );
		sensitivity[i] = gtk_vscale_new( GTK_ADJUSTMENT( sens_adj[i] ) );
		gtk_scale_set_draw_value( GTK_SCALE( sensitivity[i] ) , FALSE );
		gtk_widget_show( sensitivity[i] );
		gtk_box_pack_start( GTK_BOX( pack1 ), sensitivity[i],
				FALSE, FALSE, FALSE );
		progress[i] = gtk_progress_bar_new();
		gtk_progress_bar_set_orientation( GTK_PROGRESS_BAR( progress[i] ),
					      GTK_PROGRESS_BOTTOM_TO_TOP );
		gtk_box_pack_start( GTK_BOX( pack1 ), progress[i],
				  FALSE, FALSE, FALSE );
		gtk_widget_show( progress[i] );
	}
	
	gtk_box_pack_end( GTK_BOX(pack2), pack1, FALSE, FALSE, FALSE );
	gtk_widget_show( pack1 );
	gtk_widget_show( pack2 );

	gtk_container_add( GTK_CONTAINER( window ), pack2 );
	
	gtk_signal_connect( GTK_OBJECT( window ), "delete_event",
			    GTK_SIGNAL_FUNC( delete_event ), NULL );

	gtk_signal_connect( GTK_OBJECT( window ), "destroy",
			    GTK_SIGNAL_FUNC( destroy ), NULL );

	gtk_widget_show( window );

	if( audio_start( "localhost", 0, 1024, 22000 ) == -1 ) {
		return -1;
	}

	fft_start( 1024, 22000 );

	dammit = open( "/dev/dammit0", O_WRONLY );
	if( dammit < 0 ) {
		printf("Error opening dammit\n");
		return 1;
	}
	timeout = gtk_timeout_add( 30, do_analyze, NULL );
	idle_id = gtk_idle_add( update_audio, 0);
	gtk_main();
	fft_stop();
	audio_stop();
	close( dammit );
	return 0;
}
